@@ -20,13 +20,12 @@
   Text .frm files management routines
 */
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "parse_file.h"
 #include "unireg.h"                            // CREATE_MODE
 #include "sql_table.h"                        // build_table_filename
 #include <m_ctype.h>
-#include <my_sys.h>
 #include <my_dir.h>
 
 /* from sql_db.cc */
@@ -136,7 +135,7 @@ static ulonglong view_algo_from_frm(ulonglong val)
 
 
 static my_bool
-write_parameter(IO_CACHE *file, uchar* base, File_option *parameter)
+write_parameter(IO_CACHE *file, const uchar* base, File_option *parameter)
 {
   char num_buf[20];			// buffer for numeric operations
   // string for numeric operations
@@ -248,8 +247,9 @@ write_parameter(IO_CACHE *file, uchar* base, File_option *parameter)
 
 
 my_bool
-sql_create_definition_file(const LEX_STRING *dir, const LEX_STRING *file_name,
-			   const LEX_STRING *type,
+sql_create_definition_file(const LEX_CSTRING *dir,
+                           const LEX_CSTRING *file_name,
+			   const LEX_CSTRING *type,
 			   uchar* base, File_option *parameters)
 {
   File handler;
@@ -399,7 +399,7 @@ my_bool rename_in_schema_file(THD *thd,
 */
 
 File_parser * 
-sql_parse_prepare(const LEX_STRING *file_name, MEM_ROOT *mem_root,
+sql_parse_prepare(const LEX_CSTRING *file_name, MEM_ROOT *mem_root,
 		  bool bad_format_errors)
 {
   MY_STAT stat_info;
@@ -483,8 +483,7 @@ frm_error:
     my_error(ER_FPARSER_BAD_HEADER, MYF(0), file_name->str);
     DBUG_RETURN(0);
   }
-  else
-    DBUG_RETURN(parser); // upper level have to check parser->ok()
+  DBUG_RETURN(parser); // upper level have to check parser->ok()
 }
 
 
@@ -598,13 +597,13 @@ read_escaped_string(const char *ptr, const char *eol, LEX_STRING *str)
 
 const char *
 parse_escaped_string(const char *ptr, const char *end, MEM_ROOT *mem_root,
-                     LEX_STRING *str)
+                     LEX_CSTRING *str)
 {
   const char *eol= strchr(ptr, '\n');
 
   if (eol == 0 || eol >= end ||
       !(str->str= (char*) alloc_root(mem_root, (eol - ptr) + 1)) ||
-      read_escaped_string(ptr, eol, str))
+      read_escaped_string(ptr, eol, (LEX_STRING*) str))
     return 0;
     
   return eol+1;
@@ -760,12 +759,12 @@ File_parser::parse(uchar* base, MEM_ROOT *mem_root,
     {
       File_option *parameter= parameters+first_param,
 	*parameters_end= parameters+required;
-      int len= 0;
+      size_t len= 0;
       for (; parameter < parameters_end; parameter++)
       {
 	len= parameter->name.length;
 	// check length
-	if (len < (end-ptr) && ptr[len] != '=')
+	if (len < (size_t)(end-ptr) && ptr[len] != '=')
 	  continue;
 	// check keyword
 	if (memcmp(parameter->name.str, ptr, len) == 0)
@@ -802,7 +801,7 @@ File_parser::parse(uchar* base, MEM_ROOT *mem_root,
 	case FILE_OPTIONS_ESTRING:
 	{
 	  if (!(ptr= parse_escaped_string(ptr, end, mem_root,
-					  (LEX_STRING *)
+					  (LEX_CSTRING *)
 					  (base + parameter->offset))))
 	  {
 	    my_error(ER_FPARSER_ERROR_IN_PARAMETER, MYF(0),
