@@ -471,7 +471,7 @@ bool Sql_cmd_alter_table::execute(THD *thd)
   if (check_grant(thd, priv_needed, first_table, FALSE, UINT_MAX, FALSE))
     DBUG_RETURN(TRUE);                  /* purecov: inspected */
 #ifdef WITH_WSREP
-  if (WSREP(thd) && WSREP_CLIENT(thd) &&
+  if (WSREP(thd) &&
       (!thd->is_current_stmt_binlog_format_row() ||
        !thd->find_temporary_table(first_table)))
   {
