@@ -5,7 +5,7 @@
 /*                                                                     */
 /* COPYRIGHT:                                                          */
 /* ----------                                                          */
-/*  (C) Copyright to the author Olivier BERTRAND          1998-2018    */
+/*  (C) Copyright to the author Olivier BERTRAND          1998-2020    */
 /*                                                                     */
 /* WHAT THIS PROGRAM DOES:                                             */
 /* -----------------------                                             */
@@ -39,12 +39,12 @@
 /***********************************************************************/
 #include "my_global.h"
 #include "my_pthread.h"
-#if defined(__WIN__)
+#if defined(_WIN32)
 #include <io.h>
 #include <fcntl.h>
 #include <errno.h>
 #define BIGMEM         1048576            // 1 Megabyte
-#else     // !__WIN__
+#else     // !_WIN32
 #include <unistd.h>
 #include <fcntl.h>
 //#if defined(THREAD)
@@ -52,7 +52,7 @@
 //#endif   // THREAD
 #include <stdarg.h>
 #define BIGMEM      2147483647            // Max int value
-#endif    // !__WIN__
+#endif    // !_WIN32
 #include <locale.h>
 
 /***********************************************************************/
@@ -89,11 +89,11 @@ extern "C" {
 extern char version[];
 } // extern "C"
 
-//#if defined(__WIN__)
+//#if defined(_WIN32)
 //extern CRITICAL_SECTION parsec;      // Used calling the Flex parser
-//#else   // !__WIN__
+//#else   // !_WIN32
 extern pthread_mutex_t parmut;
-//#endif  // !__WIN__
+//#endif  // !_WIN32
 
 // The debug trace used by the main thread
 FILE *pfile = NULL;
@@ -386,11 +386,11 @@ char *SetPath(PGLOBAL g, const char *path)
 		} // endif path
 
 		if (*path != '.') {
-#if defined(__WIN__)
+#if defined(_WIN32)
 			const char *s = "\\";
-#else   // !__WIN__
+#else   // !_WIN32
 			const char *s = "/";
-#endif  // !__WIN__
+#endif  // !_WIN32
 			strcat(strcat(strcat(strcpy(buf, "."), s), path), s);
 		} else
 			strcpy(buf, path);
@@ -409,7 +409,7 @@ char *ExtractFromPath(PGLOBAL g, char *pBuff, char *FileName, OPVAL op)
   char *drive = NULL, *direc = NULL, *fname = NULL, *ftype = NULL;
 
   switch (op) {           // Determine which part to extract
-#if defined(__WIN__)
+#if defined(_WIN32)
     case OP_FDISK: drive = pBuff; break;
 #endif   // !UNIX
     case OP_FPATH: direc = pBuff; break;
@@ -1244,14 +1244,14 @@ void *PlgDBalloc(PGLOBAL g, void *area, MBLOCK& mp)
   mp.Sub = mp.Size <= ((mp.Sub) ? maxsub : (maxsub >> 2));
 
 	if (trace(2))
-		htrc("PlgDBalloc: in %p size=%d used=%d free=%d sub=%d\n",
+		htrc("PlgDBalloc: in %p size=%zd used=%zd free=%zd sub=%d\n",
 			arp, mp.Size, pph->To_Free, pph->FreeBlk, mp.Sub);
 
 	if (!mp.Sub) {
     // For allocations greater than one fourth of remaining storage
     // in the area, do allocate from virtual storage.
 		const char*v = "malloc";
-#if defined(__WIN__)
+#if defined(_WIN32)
 		if (mp.Size >= BIGMEM) {
 			v = "VirtualAlloc";
 			mp.Memp = VirtualAlloc(NULL, mp.Size, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
@@ -1260,7 +1260,7 @@ void *PlgDBalloc(PGLOBAL g, void *area, MBLOCK& mp)
       mp.Memp = malloc(mp.Size);
 
 		if (trace(8))
-			htrc("PlgDBalloc: %s(%d) at %p\n", v, mp.Size, mp.Memp);
+			htrc("PlgDBalloc: %s(%zd) at %p\n", v, mp.Size, mp.Memp);
 
 		if (!mp.Inlist && mp.Memp) {
       // New allocated block, put it in the memory block chain.
@@ -1292,7 +1292,7 @@ void *PlgDBrealloc(PGLOBAL g, void *area, MBLOCK& mp, size_t newsize)
 #endif
 
   if (trace(2))
-    htrc("PlgDBrealloc: %p size=%d sub=%d\n", mp.Memp, mp.Size, mp.Sub);
+    htrc("PlgDBrealloc: %p size=%zd sub=%d\n", mp.Memp, mp.Size, mp.Sub);
 
   if (newsize == mp.Size)
     return mp.Memp;      // Nothing to do
@@ -1342,7 +1342,7 @@ void *PlgDBrealloc(PGLOBAL g, void *area, MBLOCK& mp, size_t newsize)
   } // endif's
 
   if (trace(8))
-    htrc(" newsize=%d newp=%p sub=%d\n", mp.Size, mp.Memp, mp.Sub);
+    htrc(" newsize=%zd newp=%p sub=%d\n", mp.Size, mp.Memp, mp.Sub);
 
   return mp.Memp;
   } // end of PlgDBrealloc
@@ -1354,7 +1354,7 @@ void PlgDBfree(MBLOCK& mp)
   {
 	if (!mp.Sub && mp.Memp) {
 		const char*v = "free";
-#if defined(__WIN__)
+#if defined(_WIN32)
 		if (mp.Size >= BIGMEM) {
 			v = "VirtualFree";
 			VirtualFree(mp.Memp, 0, MEM_RELEASE);
@@ -1394,13 +1394,13 @@ void *PlgDBSubAlloc(PGLOBAL g, void *memp, size_t size)
   pph = (PPOOLHEADER)memp;
 
   if (trace(16))
-    htrc("PlgDBSubAlloc: memp=%p size=%d used=%d free=%d\n",
+    htrc("PlgDBSubAlloc: memp=%p size=%zd used=%zd free=%zd\n",
          memp, size, pph->To_Free, pph->FreeBlk);
 
-  if ((uint)size > pph->FreeBlk) {   /* Not enough memory left in pool */
+  if (size > pph->FreeBlk) {   /* Not enough memory left in pool */
     sprintf(g->Message,
-    "Not enough memory in Work area for request of %d (used=%d free=%d)",
-            (int) size, pph->To_Free, pph->FreeBlk);
+    "Not enough memory in Work area for request of %zd (used=%zd free=%zd)",
+            size, pph->To_Free, pph->FreeBlk);
 
     if (trace(1))
       htrc("%s\n", g->Message);
@@ -1416,7 +1416,7 @@ void *PlgDBSubAlloc(PGLOBAL g, void *memp, size_t size)
   pph->FreeBlk -= size;                 // New size   of pool free block
 
   if (trace(16))
-    htrc("Done memp=%p used=%d free=%d\n",
+    htrc("Done memp=%p used=%zd free=%zd\n",
          memp, pph->To_Free, pph->FreeBlk);
 
   return (memp);
@@ -1556,11 +1556,11 @@ int FileComp(PGLOBAL g, char *file1, char *file2)
   bp[0] = buff1; bp[1] = buff2;
 
   for (i = 0; i < 2; i++) {
-#if defined(__WIN__)
+#if defined(_WIN32)
     h[i]= global_open(g, MSGID_NONE, fn[i], _O_RDONLY | _O_BINARY);
-#else   // !__WIN__
+#else   // !_WIN32
     h[i]= global_open(g, MSGOD_NONE, fn[i], O_RDONLY);
-#endif  // !__WIN__
+#endif  // !_WIN32
 
     if (h[i] == -1) {
 //      if (errno != ENOENT) {
