@@ -194,7 +194,7 @@ PQRYRES TabColumns(PGLOBAL g, THD *thd, const char *db,
 
     // Get column name
     crp = qrp->Colresp;                    // Column_Name
-    colname = (char *)fp->field_name;
+    colname = (char *)fp->field_name.str;
     crp->Kdata->SetValue(colname, i);
 
 //  chset = (char *)fp->charset()->name;
@@ -276,7 +276,7 @@ PQRYRES TabColumns(PGLOBAL g, THD *thd, const char *db,
     crp = crp->Next;                       // Remark
 
     // For Valgrind
-    if (fp->comment.length > 0 && (fld = fp->comment.str))
+    if (fp->comment.length > 0 && (fld = (char*) fp->comment.str))
       crp->Kdata->SetValue(fld, fp->comment.length, i);
     else
       crp->Kdata->Reset(i);
@@ -708,7 +708,7 @@ bool PRXCOL::Init(PGLOBAL g, PTDB tp)
     MODE mode = To_Tdb->GetMode();
 
     // Needed for MYSQL subtables
-    ((XCOLBLK*)Colp)->Name = Decode(g, Colp->GetName());
+    ((COLBLK*)Colp)->SetName(Decode(g, Colp->GetName()));
 
     // May not have been done elsewhere
     Colp->InitValue(g);        
