@@ -40,6 +40,45 @@ to data_error. */
 ut_d(byte data_error);
 #endif /* UNIV_DEBUG */
 
+/** Trim the tail of an index tuple before insert or update.
+After instant ADD COLUMN, if the last fields of a clustered index tuple
+match the default values that were explicitly specified or implied during
+ADD COLUMN, there will be no need to store them.
+NOTE: A page latch in the index must be held, so that the index
+may not lose 'instantness' before the trimmed tuple has been
+inserted or updated.
+@param[in]	index	index possibly with instantly added columns */
+void dtuple_t::trim(const dict_index_t& index)
+{
+	ut_ad(n_fields >= index.n_core_fields);
+	ut_ad(n_fields <= index.n_fields);
+	ut_ad(index.is_instant());
+
+	ulint i = n_fields;
+	for (; i > index.n_core_fields; i--) {
+		const dfield_t* dfield = dtuple_get_nth_field(this, i - 1);
+		const dict_col_t* col = dict_index_get_nth_col(&index, i - 1);
+
+		if (col->is_dropped()) {
+			continue;
+		}
+
+		ut_ad(col->is_added());
+		ulint len = dfield_get_len(dfield);
+		if (len != col->def_val.len) {
+			break;
+		}
+
+		if (len != 0 && len != UNIV_SQL_NULL
+		    && dfield->data != col->def_val.data
+		    && memcmp(dfield->data, col->def_val.data, len)) {
+			break;
+		}
+	}
+
+	n_fields = i;
+}
+
 /** Compare two data tuples.
 @param[in] tuple1 first data tuple
 @param[in] tuple2 second data tuple
@@ -195,7 +234,7 @@ dtuple_validate(
 	const dtuple_t*	tuple)	/*!< in: tuple */
 {
 	ut_ad(tuple->magic_n == DATA_TUPLE_MAGIC_N);
-#ifdef HAVE_valgrind_or_MSAN
+#ifdef HAVE_valgrind
 	const ulint n_fields = dtuple_get_n_fields(tuple);
 
 	for (ulint i = 0; i < n_fields; i++) {
@@ -206,7 +245,7 @@ dtuple_validate(
 					  dfield_get_len(field));
 		}
 	}
-#endif /* HAVE_valgrind_or_MSAN */
+#endif /* HAVE_valgrind */
 	ut_ad(dtuple_check_typed(tuple));
 
 	return(TRUE);
@@ -288,7 +327,7 @@ dfield_print_also_hex(
 			val = mach_read_from_1(data);
 
 			if (!(prtype & DATA_UNSIGNED)) {
-				val &= ~0x80;
+				val &= ~0x80U;
 				fprintf(stderr, "%ld", (long) val);
 			} else {
 				fprintf(stderr, "%lu", (ulong) val);
@@ -299,7 +338,7 @@ dfield_print_also_hex(
 			val = mach_read_from_2(data);
 
 			if (!(prtype & DATA_UNSIGNED)) {
-				val &= ~0x8000;
+				val &= ~0x8000U;
 				fprintf(stderr, "%ld", (long) val);
 			} else {
 				fprintf(stderr, "%lu", (ulong) val);
@@ -310,7 +349,7 @@ dfield_print_also_hex(
 			val = mach_read_from_3(data);
 
 			if (!(prtype & DATA_UNSIGNED)) {
-				val &= ~0x800000;
+				val &= ~0x800000U;
 				fprintf(stderr, "%ld", (long) val);
 			} else {
 				fprintf(stderr, "%lu", (ulong) val);
@@ -539,7 +578,6 @@ dtuple_convert_big_rec(
 	mem_heap_t*	heap;
 	big_rec_t*	vector;
 	dfield_t*	dfield;
-	dict_field_t*	ifield;
 	ulint		size;
 	ulint		n_fields;
 	ulint		local_prefix_len;
@@ -548,7 +586,14 @@ dtuple_convert_big_rec(
 		return(NULL);
 	}
 
-	const ulint local_len = index->table->get_overflow_field_local_len();
+	if (!index->table->space) {
+		return NULL;
+	}
+
+	ulint local_len = index->table->get_overflow_field_local_len();
+	const auto zip_size = index->table->space->zip_size();
+
+	ut_ad(index->n_uniq > 0);
 
 	ut_a(dtuple_check_typed_no_assert(entry));
 
@@ -571,24 +616,42 @@ dtuple_convert_big_rec(
 	stored externally */
 
 	n_fields = 0;
+	ulint longest_i;
+
+	const bool mblob = entry->is_alter_metadata();
+	ut_ad(entry->n_fields >= index->first_user_field() + mblob);
+	ut_ad(entry->n_fields - mblob <= index->n_fields);
+
+	if (mblob) {
+		longest_i = index->first_user_field();
+		dfield = dtuple_get_nth_field(entry, longest_i);
+		local_len = BTR_EXTERN_FIELD_REF_SIZE;
+		ut_ad(!dfield_is_ext(dfield));
+		goto ext_write;
+	}
+
+	if (!dict_table_has_atomic_blobs(index->table)) {
+		/* up to MySQL 5.1: store a 768-byte prefix locally */
+		local_len = BTR_EXTERN_FIELD_REF_SIZE
+			+ DICT_ANTELOPE_MAX_INDEX_COL_LEN;
+	} else {
+		/* new-format table: do not store any BLOB prefix locally */
+		local_len = BTR_EXTERN_FIELD_REF_SIZE;
+	}
 
 	while (page_zip_rec_needs_ext(rec_get_converted_size(index, entry,
 							     *n_ext),
-				      dict_table_is_comp(index->table),
+				      index->table->not_redundant(),
 				      dict_index_get_n_fields(index),
-				      dict_table_page_size(index->table))) {
-
-		ulint			i;
-		ulint			longest		= 0;
-		ulint			longest_i	= ULINT_MAX;
-		byte*			data;
-
-		for (i = dict_index_get_n_unique_in_tree(index);
-		     i < dtuple_get_n_fields(entry); i++) {
+				      zip_size)) {
+		longest_i = 0;
+		for (ulint i = index->first_user_field(), longest = 0;
+		     i + mblob < entry->n_fields; i++) {
 			ulint	savings;
+			dfield = dtuple_get_nth_field(entry, i + mblob);
 
-			dfield = dtuple_get_nth_field(entry, i);
-			ifield = dict_index_get_nth_field(index, i);
+			const dict_field_t* ifield = dict_index_get_nth_field(
+				index, i);
 
 			/* Skip fixed-length, NULL, externally stored,
 			or short columns */
@@ -623,14 +686,14 @@ dtuple_convert_big_rec(
 				goto skip_field;
 			}
 
-			longest_i = i;
+			longest_i = i + mblob;
 			longest = savings;
 
 skip_field:
 			continue;
 		}
 
-		if (!longest) {
+		if (!longest_i) {
 			/* Cannot shorten more */
 
 			mem_heap_free(heap);
@@ -643,9 +706,8 @@ skip_field:
 		We store the first bytes locally to the record. Then
 		we can calculate all ordering fields in all indexes
 		from locally stored data. */
-
 		dfield = dtuple_get_nth_field(entry, longest_i);
-		ifield = dict_index_get_nth_field(index, longest_i);
+ext_write:
 		local_prefix_len = local_len - BTR_EXTERN_FIELD_REF_SIZE;
 
 		vector->append(
@@ -656,7 +718,8 @@ skip_field:
 				+ local_prefix_len));
 
 		/* Allocate the locally stored part of the column. */
-		data = static_cast<byte*>(mem_heap_alloc(heap, local_len));
+		byte* data = static_cast<byte*>(
+			mem_heap_alloc(heap, local_len));
 
 		/* Copy the local prefix. */
 		memcpy(data, dfield_get_data(dfield), local_prefix_len);
@@ -704,7 +767,7 @@ void
 dtuple_convert_back_big_rec(
 /*========================*/
 	dict_index_t*	index MY_ATTRIBUTE((unused)),	/*!< in: index */
-	dtuple_t*	entry,	/*!< in: entry whose data was put to vector */
+	dtuple_t*	entry,	/*!< in/out: entry whose data was put to vector */
 	big_rec_t*	vector)	/*!< in, own: big rec vector; it is
 				freed in this function */
 {
@@ -772,6 +835,7 @@ dfield_t::clone(mem_heap_t* heap) const
 	dfield_t* obj = static_cast<dfield_t*>(
 		mem_heap_alloc(heap, sizeof(dfield_t) + size));
 
+	ut_ad(len != UNIV_SQL_DEFAULT);
 	obj->ext  = ext;
 	obj->len  = len;
 	obj->type = type;
