@@ -143,7 +143,7 @@ PFS_thread *thread_array= NULL;
   File instrumentation instances array.
   @sa file_max
   @sa file_lost
-  @sa filename_hash
+  @sa pfs_filename_hash
 */
 PFS_file *file_array= NULL;
 
@@ -189,8 +189,8 @@ static unsigned char *history_stmts_digest_token_array= NULL;
 static char *thread_session_connect_attrs_array= NULL;
 
 /** Hash table for instrumented files. */
-LF_HASH filename_hash;
-/** True if filename_hash is initialized. */
+LF_HASH pfs_filename_hash;
+/** True if pfs_filename_hash is initialized. */
 static bool filename_hash_inited= false;
 
 /**
@@ -586,7 +586,7 @@ int init_file_hash(void)
 {
   if ((! filename_hash_inited) && (file_max > 0))
   {
-    lf_hash_init(&filename_hash, sizeof(PFS_file*), LF_HASH_UNIQUE,
+    lf_hash_init(&pfs_filename_hash, sizeof(PFS_file*), LF_HASH_UNIQUE,
                  0, 0, filename_hash_get_key, &my_charset_bin);
     /* filename_hash.size= file_max; */
     filename_hash_inited= true;
@@ -599,7 +599,7 @@ void cleanup_file_hash(void)
 {
   if (filename_hash_inited)
   {
-    lf_hash_destroy(&filename_hash);
+    lf_hash_destroy(&pfs_filename_hash);
     filename_hash_inited= false;
   }
 }
@@ -1186,7 +1186,7 @@ void destroy_thread(PFS_thread *pfs)
 }
 
 /**
-  Get the hash pins for @filename_hash.
+  Get the hash pins for @pfs_filename_hash.
   @param thread The running thread.
   @returns The LF_HASH pins for the thread.
 */
@@ -1196,7 +1196,7 @@ LF_PINS* get_filename_hash_pins(PFS_thread *thread)
   {
     if (! filename_hash_inited)
       return NULL;
-    thread->m_filename_hash_pins= lf_hash_get_pins(&filename_hash);
+    thread->m_filename_hash_pins= lf_hash_get_pins(&pfs_filename_hash);
   }
   return thread->m_filename_hash_pins;
 }
@@ -1302,7 +1302,7 @@ find_or_create_file(PFS_thread *thread, PFS_file_class *klass,
   *buf_end= '\0';
 
   normalized_filename= buffer;
-  size_t normalized_length= strlen(normalized_filename);
+  uint normalized_length= static_cast<uint>(strlen(normalized_filename));
 
   PFS_file **entry;
   uint retry_count= 0;
@@ -1314,7 +1314,7 @@ find_or_create_file(PFS_thread *thread, PFS_file_class *klass,
 search:
 
   entry= reinterpret_cast<PFS_file**>
-    (lf_hash_search(&filename_hash, pins,
+    (lf_hash_search(&pfs_filename_hash, pins,
                     normalized_filename, normalized_length));
   if (entry && (entry != MY_ERRPTR))
   {
@@ -1359,7 +1359,7 @@ search:
         pfs->m_identity= (const void *)pfs;
 
         int res;
-        res= lf_hash_insert(&filename_hash, thread->m_filename_hash_pins,
+        res= lf_hash_insert(&pfs_filename_hash, thread->m_filename_hash_pins,
                             &pfs);
         if (likely(res == 0))
         {
@@ -1426,7 +1426,7 @@ void destroy_file(PFS_thread *thread, PFS_file *pfs)
   LF_PINS *pins= get_filename_hash_pins(thread);
   DBUG_ASSERT(pins != NULL);
 
-  lf_hash_delete(&filename_hash, pins,
+  lf_hash_delete(&pfs_filename_hash, pins,
                  pfs->m_filename, pfs->m_filename_length);
   if (klass->is_singleton())
     klass->m_singleton= NULL;
