@@ -1654,21 +1654,18 @@ group_by_handler *spider_create_group_by_handler(
     if (from->table->part_info)
     {
       DBUG_PRINT("info",("spider partition handler"));
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
-      ha_partition *partition = (ha_partition *) from->table->file;
-      part_id_range *part_spec = partition->get_part_spec();
-      DBUG_PRINT("info",("spider part_spec->start_part=%u", part_spec->start_part));
-      DBUG_PRINT("info",("spider part_spec->end_part=%u", part_spec->end_part));
-      if (
-        part_spec->start_part == partition->get_no_current_part_id() ||
-        part_spec->start_part != part_spec->end_part
-      ) {
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
+      partition_info *part_info = from->table->part_info;
+      uint bits = bitmap_bits_set(&part_info->read_partitions);
+      DBUG_PRINT("info",("spider bits=%u", bits));
+      if (bits != 1)
+      {
         DBUG_PRINT("info",("spider using multiple partitions is not supported by this feature yet"));
 #else
         DBUG_PRINT("info",("spider partition is not supported by this feature yet"));
 #endif
         DBUG_RETURN(NULL);
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
       }
 #endif
     }
@@ -1686,17 +1683,18 @@ group_by_handler *spider_create_group_by_handler(
     /* all tables are const_table */
     DBUG_RETURN(NULL);
   }
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   if (from->table->part_info)
   {
+    partition_info *part_info = from->table->part_info;
+    uint part = bitmap_get_first_set(&part_info->read_partitions);
     ha_partition *partition = (ha_partition *) from->table->file;
-    part_id_range *part_spec = partition->get_part_spec();
     handler **handlers = partition->get_child_handlers();
-    spider = (ha_spider *) handlers[part_spec->start_part];
+    spider = (ha_spider *) handlers[part];
   } else {
 #endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
 #endif
   share = spider->share;
@@ -1717,17 +1715,18 @@ group_by_handler *spider_create_group_by_handler(
   {
     if (from->table->const_table)
       continue;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     if (from->table->part_info)
     {
+      partition_info *part_info = from->table->part_info;
+      uint part = bitmap_get_first_set(&part_info->read_partitions);
       ha_partition *partition = (ha_partition *) from->table->file;
-      part_id_range *part_spec = partition->get_part_spec();
       handler **handlers = partition->get_child_handlers();
-      spider = (ha_spider *) handlers[part_spec->start_part];
+      spider = (ha_spider *) handlers[part];
     } else {
 #endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
@@ -1755,17 +1754,18 @@ group_by_handler *spider_create_group_by_handler(
   do {
     if (from->table->const_table)
       continue;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     if (from->table->part_info)
     {
+      partition_info *part_info = from->table->part_info;
+      uint part = bitmap_get_first_set(&part_info->read_partitions);
       ha_partition *partition = (ha_partition *) from->table->file;
-      part_id_range *part_spec = partition->get_part_spec();
       handler **handlers = partition->get_child_handlers();
-      spider = (ha_spider *) handlers[part_spec->start_part];
+      spider = (ha_spider *) handlers[part];
     } else {
 #endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
@@ -1908,17 +1908,18 @@ group_by_handler *spider_create_group_by_handler(
   {
     from = from->next_local;
   }
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   if (from->table->part_info)
   {
+    partition_info *part_info = from->table->part_info;
+    uint part = bitmap_get_first_set(&part_info->read_partitions);
     ha_partition *partition = (ha_partition *) from->table->file;
-    part_id_range *part_spec = partition->get_part_spec();
     handler **handlers = partition->get_child_handlers();
-    spider = (ha_spider *) handlers[part_spec->start_part];
+    spider = (ha_spider *) handlers[part];
   } else {
 #endif
     spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
   }
 #endif
   share = spider->share;
@@ -1996,17 +1997,18 @@ group_by_handler *spider_create_group_by_handler(
       continue;
     fields->clear_conn_holder_from_conn();
 
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     if (from->table->part_info)
     {
+      partition_info *part_info = from->table->part_info;
+      uint part = bitmap_get_first_set(&part_info->read_partitions);
       ha_partition *partition = (ha_partition *) from->table->file;
-      part_id_range *part_spec = partition->get_part_spec();
       handler **handlers = partition->get_child_handlers();
-      spider = (ha_spider *) handlers[part_spec->start_part];
+      spider = (ha_spider *) handlers[part];
     } else {
 #endif
       spider = (ha_spider *) from->table->file;
-#if defined(PARTITION_HAS_GET_CHILD_HANDLERS) && defined(PARTITION_HAS_GET_PART_SPEC)
+#if defined(PARTITION_HAS_GET_CHILD_HANDLERS)
     }
 #endif
     share = spider->share;
